@@ -4,6 +4,8 @@ import sys as _sys
 
 __all__ = ['Enum', 'IntEnum', 'unique']
 
+version = 1, 0, 3
+
 pyver = float('%s.%s' % _sys.version_info[:2])
 
 try:
@@ -80,7 +82,6 @@ def _make_class_unpicklable(cls):
     def _break_on_call_reduce(self, protocol=None):
         raise TypeError('%r cannot be pickled' % self)
     cls.__reduce_ex__ = _break_on_call_reduce
-    #cls.__reduce__ = _break_on_call_reduce
     cls.__module__ = '<unknown>'
 
 
@@ -152,10 +153,6 @@ class EnumMeta(type):
                 classdict[k] = v
 
         member_type, first_enum = metacls._get_mixins_(bases)
-        #if member_type is object:
-        #    use_args = False
-        #else:
-        #    use_args = True
         __new__, save_new, use_args = metacls._find_new_(classdict, member_type,
                                                         first_enum)
         # save enum items into separate mapping so they don't get baked into
@@ -167,14 +164,15 @@ class EnumMeta(type):
         # py2 support for definition order
         __order__ = classdict.get('__order__')
         if __order__ is None:
-            __order__ = classdict._member_names
             if pyver < 3.0:
-                order_specified = False
+                try:
+                    __order__ = [name for (name, value) in sorted(members.items(), key=lambda item: item[1])]
+                except TypeError:
+                    __order__ = [name for name in sorted(members.keys())]
             else:
-                order_specified = True
+                __order__ = classdict._member_names
         else:
             del classdict['__order__']
-            order_specified = True
             if pyver < 3.0:
                 __order__ = __order__.replace(',', ' ').split()
                 aliases = [name for name in members if name not in __order__]
@@ -189,7 +187,7 @@ class EnumMeta(type):
         # create our new Enum type
         enum_class = super(EnumMeta, metacls).__new__(metacls, cls, bases, classdict)
         enum_class._member_names_ = []               # names in random order
-        if order_specified and OrderedDict is not None:
+        if OrderedDict is not None:
             enum_class._member_map_ = OrderedDict()
         else:
             enum_class._member_map_ = {}             # name->value map
@@ -242,35 +240,38 @@ class EnumMeta(type):
             except TypeError:
                 pass
 
-        # in Python2.x we cannot know definition order, so go with value order
-        # unless __order__ was specified in the class definition
-        if not order_specified:
-            enum_class._member_names_ = [
-                e[0] for e in sorted(
-                [(name, enum_class._member_map_[name]) for name in enum_class._member_names_],
-                 key=lambda t: t[1]._value_
-                        )]
+
+        # If a custom type is mixed into the Enum, and it does not know how
+        # to pickle itself, pickle.dumps will succeed but pickle.loads will
+        # fail.  Rather than have the error show up later and possibly far
+        # from the source, sabotage the pickle protocol for this class so
+        # that pickle.dumps also fails.
+        #
+        # However, if the new class implements its own __reduce_ex__, do not
+        # sabotage -- it's on them to make sure it works correctly.  We use
+        # __reduce_ex__ instead of any of the others as it is preferred by
+        # pickle over __reduce__, and it handles all pickle protocols.
+        unpicklable = False
+        if '__reduce_ex__' not in classdict:
+            if member_type is not object:
+                methods = ('__getnewargs_ex__', '__getnewargs__',
+                        '__reduce_ex__', '__reduce__')
+                if not any(m in member_type.__dict__ for m in methods):
+                    _make_class_unpicklable(enum_class)
+                    unpicklable = True
+
 
         # double check that repr and friends are not the mixin's or various
         # things break (such as pickle)
-        if Enum is not None:
-            setattr(enum_class, '__getnewargs__', Enum.__getnewargs__)
-            setattr(enum_class, '__reduce_ex__', Enum.__reduce_ex__)
-        for name in ('__repr__', '__str__', '__format__', ):
+        for name in ('__repr__', '__str__', '__format__', '__reduce_ex__'):
             class_method = getattr(enum_class, name)
             obj_method = getattr(member_type, name, None)
             enum_method = getattr(first_enum, name, None)
-            if obj_method is not None and obj_method is class_method:
+            if name not in classdict and class_method is not enum_method:
+                if name == '__reduce_ex__' and unpicklable:
+                    continue
                 setattr(enum_class, name, enum_method)
 
-        # check for a __getnewargs__, and if not present sabotage
-        # pickling, since it won't work anyway
-        if member_type is not object:
-            methods = ('__getnewargs_ex__', '__getnewargs__',
-                    '__reduce_ex__', '__reduce__')
-            if not any(map(member_type.__dict__.get, methods)):
-                _make_class_unpicklable(enum_class)
-
         # method resolution and int's are not playing nice
         # Python's less than 2.6 use __cmp__
 
@@ -641,8 +642,13 @@ temp_enum_dict['__str__'] = __str__
 del __str__
 
 def __dir__(self):
-    added_behavior = [m for m in self.__class__.__dict__ if m[0] != '_']
-    return (['__class__', '__doc__', '__module__', 'name', 'value'] + added_behavior)
+    added_behavior = [
+            m
+            for cls in self.__class__.mro()
+            for m in cls.__dict__
+            if m[0] != '_'
+            ]
+    return (['__class__', '__doc__', '__module__', ] + added_behavior)
 temp_enum_dict['__dir__'] = __dir__
 del __dir__
 
@@ -716,18 +722,13 @@ def __ne__(self, other):
 temp_enum_dict['__ne__'] = __ne__
 del __ne__
 
-def __getnewargs__(self):
-    return (self._value_, )
-temp_enum_dict['__getnewargs__'] = __getnewargs__
-del __getnewargs__
-
 def __hash__(self):
     return hash(self._name_)
 temp_enum_dict['__hash__'] = __hash__
 del __hash__
 
 def __reduce_ex__(self, proto):
-    return self.__class__, self.__getnewargs__()
+    return self.__class__, (self._value_, )
 temp_enum_dict['__reduce_ex__'] = __reduce_ex__
 del __reduce_ex__
 
