@@ -95,7 +95,7 @@ def test_pickle_dump_load(assertion, source, target=None,
             else:
                 assertion(loads(dumps(source, protocol=protocol)), target)
         except Exception:
-            exc = sys.exc_info()[1]
+            exc, tb = sys.exc_info()[1:]
             failures.append('%2d: %s' %(protocol, exc))
     if failures:
         raise ValueError('Failed with protocols: %s' % ', '.join(failures))
@@ -109,7 +109,7 @@ def test_pickle_exception(assertion, exception, obj,
             assertion(exception, dumps, obj, protocol=protocol)
         except Exception:
             exc = sys.exc_info()[1]
-            failures.append('%d: %s' % (protocol, exc))
+            failures.append('%d: %s %s' % (protocol, exc.__class__.__name__, exc))
     if failures:
         raise ValueError('Failed with protocols: %s' % ', '.join(failures))
 
@@ -178,7 +178,7 @@ class TestEnum(unittest.TestCase):
             IDES_OF_MARCH = 2013, 3, 15
         self.Holiday = Holiday
 
-    if pyver >= 2.6:     # cannot specify custom `dir` on this version
+    if pyver >= 2.6:     # cannot specify custom `dir` on previous versions
         def test_dir_on_class(self):
             Season = self.Season
             self.assertEqual(
@@ -194,6 +194,18 @@ class TestEnum(unittest.TestCase):
                 set(['__class__', '__doc__', '__module__', 'name', 'value']),
                 )
 
+        def test_dir_on_sub_with_behavior_on_super(self):
+            # see issue22506
+            class SuperEnum(Enum):
+                def invisible(self):
+                    return "did you see me?"
+            class SubEnum(SuperEnum):
+                sample = 5
+            self.assertEqual(
+                    set(dir(SubEnum.sample)),
+                    set(['__class__', '__doc__', '__module__', 'name', 'value', 'invisible']),
+                    )
+
     if pyver >= 2.7:    # OrderedDict first available here
         def test_members_is_ordereddict_if_ordered(self):
             class Ordered(Enum):
@@ -203,12 +215,11 @@ class TestEnum(unittest.TestCase):
                 third = 'boo'
             self.assertTrue(type(Ordered.__members__) is OrderedDict)
 
-        if pyver < 3.0:
-            def test_members_is_not_ordereddict_if_not_ordered(self):
-                class Unordered(Enum):
-                    this = 'that'
-                    these = 'those'
-                self.assertTrue(type(Unordered.__members__) is dict)
+        def test_members_is_ordereddict_if_not_ordered(self):
+            class Unordered(Enum):
+                this = 'that'
+                these = 'those'
+            self.assertTrue(type(Unordered.__members__) is OrderedDict)
 
     if pyver >= 3.0:     # all objects are ordered in Python 2.x
         def test_members_is_always_ordered(self):
@@ -403,6 +414,7 @@ class TestEnum(unittest.TestCase):
         self.assertEqual(dates[Season.AUTUMN], '1031')
 
     def test_enum_duplicates(self):
+        __order__ = "SPRING SUMMER AUTUMN WINTER"
         class Season(Enum):
             SPRING = 1
             SUMMER = 2
@@ -641,6 +653,17 @@ class TestEnum(unittest.TestCase):
                 [Season.SUMMER, Season.WINTER, Season.AUTUMN, Season.SPRING],
                 )
 
+    def test_iteration_order_with_unorderable_values(self):
+        class Complex(Enum):
+            a = complex(7, 9)
+            b = complex(3.14, 2)
+            c = complex(1, -1)
+            d = complex(-77, 32)
+        self.assertEqual(
+                list(Complex),
+                [Complex.a, Complex.b, Complex.c, Complex.d],
+                )
+
     def test_programatic_function_string(self):
         SummerMonth = Enum('SummerMonth', 'june july august')
         lst = list(SummerMonth)
@@ -1098,7 +1121,7 @@ class TestEnum(unittest.TestCase):
         globals()['NEI'] = NEI
         NI5 = NamedInt('test', 5)
         self.assertEqual(NI5, 5)
-        test_pickle_dump_load(self.assertEqual, NI5, 5)
+        test_pickle_dump_load(self.assertTrue, NI5, 5)
         self.assertEqual(NEI.y.value, 2)
         test_pickle_dump_load(self.assertTrue, NEI.y)
 
@@ -1155,9 +1178,9 @@ class TestEnum(unittest.TestCase):
             globals()['NEI'] = NEI
             NI5 = NamedInt('test', 5)
             self.assertEqual(NI5, 5)
-            test_pickle_dump_load(self.assertEqual, NI5, 5, protocol=(4, 4))
+            test_pickle_dump_load(self.assertEqual, NI5, 5, protocol=(4, HIGHEST_PROTOCOL))
             self.assertEqual(NEI.y.value, 2)
-            test_pickle_dump_load(self.assertTrue, NEI.y, protocol=(4, 4))
+            test_pickle_dump_load(self.assertTrue, NEI.y, protocol=(4, HIGHEST_PROTOCOL))
 
     def test_subclasses_with_reduce(self):
         class NamedInt(int):
@@ -1271,7 +1294,7 @@ class TestEnum(unittest.TestCase):
         self.assertEqual(NEI.y.value, 2)
         test_pickle_dump_load(self.assertTrue, NEI.y)
 
-    def test_subclasses_without_getnewargs(self):
+    def test_subclasses_without_direct_pickle_support(self):
         class NamedInt(int):
             __qualname__ = 'NamedInt'
             def __new__(cls, *args):
@@ -1324,6 +1347,61 @@ class TestEnum(unittest.TestCase):
         test_pickle_exception(self.assertRaises, TypeError, NEI.x)
         test_pickle_exception(self.assertRaises, PicklingError, NEI)
 
+    def test_subclasses_without_direct_pickle_support_using_name(self):
+        class NamedInt(int):
+            __qualname__ = 'NamedInt'
+            def __new__(cls, *args):
+                _args = args
+                name, args = args[0], args[1:]
+                if len(args) == 0:
+                    raise TypeError("name and value must be specified")
+                self = int.__new__(cls, *args)
+                self._intname = name
+                self._args = _args
+                return self
+            @property
+            def __name__(self):
+                return self._intname
+            def __repr__(self):
+                # repr() is updated to include the name and type info
+                return "%s(%r, %s)" % (type(self).__name__,
+                                             self.__name__,
+                                             int.__repr__(self))
+            def __str__(self):
+                # str() is unchanged, even if it relies on the repr() fallback
+                base = int
+                base_str = base.__str__
+                if base_str.__objclass__ is object:
+                    return base.__repr__(self)
+                return base_str(self)
+            # for simplicity, we only define one operator that
+            # propagates expressions
+            def __add__(self, other):
+                temp = int(self) + int( other)
+                if isinstance(self, NamedInt) and isinstance(other, NamedInt):
+                    return NamedInt(
+                        '(%s + %s)' % (self.__name__, other.__name__),
+                        temp )
+                else:
+                    return temp
+
+        class NEI(NamedInt, Enum):
+            __qualname__ = 'NEI'
+            x = ('the-x', 1)
+            y = ('the-y', 2)
+            def __reduce_ex__(self, proto):
+                return getattr, (self.__class__, self._name_)
+
+        self.assertTrue(NEI.__new__ is Enum.__new__)
+        self.assertEqual(repr(NEI.x + NEI.y), "NamedInt('(the-x + the-y)', 3)")
+        globals()['NamedInt'] = NamedInt
+        globals()['NEI'] = NEI
+        NI5 = NamedInt('test', 5)
+        self.assertEqual(NI5, 5)
+        self.assertEqual(NEI.y.value, 2)
+        test_pickle_dump_load(self.assertTrue, NEI.y)
+        test_pickle_dump_load(self.assertTrue, NEI)
+
     def test_tuple_subclass(self):
         class SomeTuple(tuple, Enum):
             __qualname__ = 'SomeTuple'
